@@ -111,6 +111,10 @@ SEXP _RxODE_setProgSupported(SEXP);
 SEXP _RxODE_getProgSupported();
 SEXP _RxODE_rxSetIni0(SEXP);
 SEXP _RxODE_rxSetSilentErr(SEXP silentSEXP);
+SEXP _RxODE_rxUnloadAll();
+SEXP _RxODE_rxLock(SEXP);
+SEXP _RxODE_rxUnlock(SEXP);
+SEXP _RxODE_rxAllowUnload(SEXP);
 
 extern int rxIsCurrentC(SEXP obj);
 
@@ -119,7 +123,6 @@ rx_solve *getRxSolve_();
 // Remove these functions later...
 
 void rxOptionsIni();
-void rxOptionsIniData();
 /* void rxOptionsIniFocei(); */
 
 double solveLinB(rx_solve *rx, unsigned int id, double t, int linCmt,
@@ -148,7 +151,6 @@ extern void rxSingleSolve(int subid, double *_theta, double *timep,
 			  int *slvr_counter, int *dadt_counter, int *jac_counter,
 			  double *InfusionRate, int *BadDose, int *idose,
 			  double *scale, int *stateIgnore, double *mtime);
-void rxOptionsIniEnsure(int mx);
 
 void R_init_RxODE(DllInfo *info){
   R_CallMethodDef callMethods[]  = {
@@ -211,6 +213,10 @@ void R_init_RxODE(DllInfo *info){
     {"_RxODE_atolRtolFactor_", (DL_FUNC) &_RxODE_atolRtolFactor_, 1},
     {"_RxODE_rxSetIni0", (DL_FUNC) &_RxODE_rxSetIni0, 1},
     {"_RxODE_rxSetSilentErr", (DL_FUNC) &_RxODE_rxSetSilentErr, 1},
+    {"_RxODE_rxUnloadAll", (DL_FUNC) &_RxODE_rxUnloadAll, 0},
+    {"_RxODE_rxLock", (DL_FUNC) &_RxODE_rxLock, 1},
+    {"_RxODE_rxUnlock", (DL_FUNC) &_RxODE_rxUnlock, 1},
+    {"_RxODE_rxAllowUnload", (DL_FUNC) &_RxODE_rxAllowUnload, 1},
     {NULL, NULL, 0}
   };
   // C callable to assign environments.
@@ -242,7 +248,6 @@ void R_init_RxODE(DllInfo *info){
   R_RegisterCCallable("RxODE","RxODE_current_fn_pointer_id", (DL_FUNC) &RxODE_current_fn_pointer_id);
   R_RegisterCCallable("RxODE","getRxSolve_", (DL_FUNC) &getRxSolve_);
   R_RegisterCCallable("RxODE", "rxSingleSolve", (DL_FUNC) &rxSingleSolve);
-  R_RegisterCCallable("RxODE", "rxOptionsIniEnsure0", (DL_FUNC) &rxOptionsIniEnsure);
   
   static const R_CMethodDef cMethods[] = {
     {"RxODE_sum",               (DL_FUNC) &RxODE_sum, 2, RxODE_Sum_t},
@@ -253,19 +258,17 @@ void R_init_RxODE(DllInfo *info){
   R_registerRoutines(info, cMethods, callMethods, NULL, NULL);
   R_useDynamicSymbols(info, FALSE);
   rxOptionsIni();
-  rxOptionsIniData();
   /* rxOptionsIniFocei(); */
 }
 
-void parseFree();
+void parseFree(int last);
 void rxOptionsFree();
 void gFree();
-/* void rxOptionsFreeFocei(); */
+void rxFreeLast();
 void R_unload_RxODE(DllInfo *info){
   gFree();
   rxOptionsFree();
   rxOptionsIni();
-  rxOptionsIniData();
-  parseFree();
-  /* rxOptionsFreeFocei(); */
+  parseFree(1);
+  rxFreeLast();
 }
